/**
 * @author Rémi Colet - cr174388 <remi.colet@icloud.com>
 */

// Create a WebSocket connection to the server
const ws = new WebSocket("ws://" + window.location.host + "/socket");
let canvas;
let canvasWidth;
let canvasHeight;
let ctx;
let obstaclesCoordinates = [];
let drawings = [];
let canvasVehiclePosition = {
    width: 0,
    height: 0,
    xPosition: 0,
    yPosition: 0
}

// Connection au websocket
ws.onopen = (event) => {
    console.log("We are connected.");
    initCanvas();
};

ws.onmessage = (event) => {
    let data = JSON.parse(event.data);

    updateVehiclePosition(data);
    drawObstacles(data);

    console.log(data);
};


function initCanvas() {
    canvas = document.getElementById("canvas");
    canvasWidth = canvas.width;
    canvasHeight = canvas.height;
    ctx = canvas.getContext("2d");
    canvasVehiclePosition = {
        width: 10,
        height: 10,
        xPosition: (canvasWidth / 2),
        yPosition: (canvasHeight / 2),
        radianAngle: 0.0
    }

    drawNewVehiclePosition(canvasVehiclePosition);
}

function drawNewVehiclePosition(currentVehiclePosition) {
    draw(pushDrawing("RECTANGLE", "#3aaaee", currentVehiclePosition.xPosition, currentVehiclePosition.yPosition, currentVehiclePosition.height, currentVehiclePosition.width));
}

function clearVehiclePreviousPosition(previousVehiclePosition) {
    draw(pushDrawing("CLEAR", "#FFFFFF", previousVehiclePosition.xPosition, previousVehiclePosition.yPosition, previousVehiclePosition.height, previousVehiclePosition.width));
}

function updateVehiclePosition(vehicleMessage) {

	// conversion de l'ange en radians
    let angleInRadian = vehicleMessage.orientation * (Math.PI / 180);

    // calcul de la distance parcouru depuis la derniere trame
    // les données sont récupérés une fois par seconde, vehicleMessage.speed est en metres/secondes
    // -> 50 px = 1m 
    let distanceFromSpeed = vehicleMessage.speed * 50; 

    // projection du deplacement sur l'axe des X
    let newXPosition = Math.trunc(canvasVehiclePosition.xPosition + (distanceFromSpeed * Math.cos(angleInRadian)));
    // projection du deplacement sur l'axe des Y
    let newYPosition = Math.trunc(canvasVehiclePosition.yPosition + (distanceFromSpeed * Math.sin(angleInRadian)));

    canvasVehiclePosition.xPosition = newXPosition;
    canvasVehiclePosition.yPosition = newYPosition;

    drawNewVehiclePosition(canvasVehiclePosition);
}

function drawObstacles(vehicleMessage) {

	// conversion de l'ange en radians
    let angleInRadian = vehicleMessage.orientation * (Math.PI / 180);

	// Si un obstacle se trouve face a la voiture
    if (vehicleMessage.distance && vehicleMessage.distance > 0.0) {

    	//calcul des coordonnées de l'obstacle (projection sur l'axe des X)
        let xObstaclePosition = canvasVehiclePosition.xPosition + ((vehicleMessage.distance * 50) * Math.cos(angleInRadian));
    	//calcul des coordonnées de l'obstacle (projection sur l'axe des Y)
        let yObstaclePosition = canvasVehiclePosition.yPosition + ((vehicleMessage.distance * 50) * Math.sin(angleInRadian));

        let obstacleCoordinate = { x: xObstaclePosition, y: yObstaclePosition };

        draw(pushDrawing("RECTANGLE", "#000000", obstacleCoordinate.x, obstacleCoordinate.y, 5, 5));

        // Si il existe des obstacles
        if (obstaclesCoordinates[obstaclesCoordinates.length - 1]) {

            // Si le precedent obstacle recontré est a moins de ~ 40cm de l'obstacle actuelement recontré
            if (Math.abs(obstaclesCoordinates[obstaclesCoordinates.length - 1].x - obstacleCoordinate.x) < 20 &&
                Math.abs(obstaclesCoordinates[obstaclesCoordinates.length - 1].y - obstacleCoordinate.y) < 20) {

            	//Alors on considere que les deux obstacles sont liés, on relie les deux par un trait 
                let linesDrawings = [];
                linesDrawings.push(pushDrawing("LINE", "#000000", obstaclesCoordinates[obstaclesCoordinates.length - 1].x, obstaclesCoordinates[obstaclesCoordinates.length - 1].y, null, null));
                linesDrawings.push(pushDrawing("LINE", "#000000", obstacleCoordinate.x, obstacleCoordinate.y, null, null));

                draw(linesDrawings);
            }
        }

        obstaclesCoordinates.push(obstacleCoordinate);
    }
}


/*
*	Verifie que les coordonnées X et Y sont comprises dans le canvas
*	Si ce n'est pas le cas, on agrandit le canvas et actualise les 
* 	coordonnées de tous les dessins precedemment realisé
*	puis on redessine l'ensemble
*/
function updateCanvasSize(xCoord, yCoord) {

    if (xCoord > canvasWidth) {
        canvasWidth = canvasWidth + 100;
        document.getElementById("canvas").width = canvasWidth;
        redraw();
    } else if (yCoord > canvasHeight) {
        canvasHeight = canvasHeight + 100;
        document.getElementById("canvas").height = canvasHeight;
        redraw();
    } else if (xCoord < 0) {
        canvasWidth = canvasWidth + 100;
        document.getElementById("canvas").width = canvasWidth;

        canvasVehiclePosition.xPosition = canvasVehiclePosition.xPosition + 100;


        drawings.forEach(function(drawing) {
            drawing.xPosition = drawing.xPosition = drawing.xPosition + 100;
        });

        redraw();

    } else if (yCoord < 0) {
        canvasHeight = canvasHeight + 100;
        document.getElementById("canvas").height = canvasHeight;

        canvasVehiclePosition.yPosition = canvasVehiclePosition.yPosition + 100;

        drawings.forEach(function(drawing) {
            drawing.yPosition = drawing.yPosition = drawing.yPosition + 100;
        });

        redraw();
    }
}

/*
*	Redessine tous les dessins que le client a recu depuis sa connection
*/
function redraw() {

    for (var i = 0; i < drawings.length; i++) {

        if (drawings[i].type === "RECTANGLE") {
            draw(drawings[i]);
        } else if (drawings[i].type === "LINE") {
            draw([drawings[i], drawings[i + 1]]);
			i = i+1;            
        }
    }
}


/*
*	Desinne le dessin drawing
*/
function draw(drawing) {

	let xCoord = drawing ? (drawing.length > 0 ? drawing[1].xPosition : drawing.xPosition) : 0;
	let yCoord = drawing ? (drawing.length > 0 ? drawing[1].yPosition : drawing.yPosition) : 0;

	// Verification, si les coordonnées du drawing ne sont plus dans le canvas, on resize le canvas auparavant
	updateCanvasSize(xCoord, yCoord);

    if (drawing) {
        if (drawing.length > 1) {

            ctx.beginPath();
            ctx.moveTo(drawing[0].xPosition, drawing[0].yPosition);
            ctx.lineTo(drawing[1].xPosition, drawing[1].yPosition);
            ctx.closePath();
            ctx.stroke();

        } else if (drawing.type === "RECTANGLE") {
            ctx.fillStyle = drawing.color;
            ctx.fillRect(drawing.xPosition, drawing.yPosition, drawing.width, drawing.height);
        }
    }
}

/*
* Ajoute un drawing a la liste des drawings
* retrun le drawing ajouté a la liste
*/
function pushDrawing(type, color, xPosition, yPosition, width, height) {
    drawings.push({
        "type": type,
        "color": color,
        "xPosition": xPosition,
        "yPosition": yPosition,
        "width": width,
        "height": height
    });

    return drawings[drawings.length - 1];
}